@@ -5,14 +5,15 @@ require_once 'Horde/Identity.php';
 
 /**
  * Horde Kronolith driver for the Kolab IMAP Server.
- * Copyright 2004-2005 Horde Project (http://horde.org/)
+ * Copyright 2004-2006 Horde Project (http://horde.org/)
  *
- * $Horde: kronolith/lib/Driver/kolab.php,v 1.16.2.3 2005/04/28 23:05:31 selsky Exp $
+ * $Horde: kronolith/lib/Driver/kolab.php,v 1.16.2.8 2006/02/02 13:49:23 selsky Exp $
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Stuart Binge <omicron@mighty.co.za>
+ * @since   Kronolith 2.0
  * @package Kronolith
  */
 class Kronolith_Driver_kolab extends Kronolith_Driver {
@@ -20,19 +21,20 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
     /**
      * Our Kolab server connection.
      *
-     * @var object Kolab $_kc
+     * @var Kolab
      */
     var $_kolab = null;
 
     function open($calendar)
     {
-        $this->_calendar = $calendar;
+        if (empty($this->_calendar) || $this->_calendar != $calendar) {
+            $this->_calendar = $calendar;
+        }
 
         $result = $this->_connect();
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
-
         return true;
     }
 
@@ -43,12 +45,9 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
 
     function _connect()
     {
-        if (isset($this->_kolab)) {
-            return true;
+        if (!isset($this->_kolab)) {
+            $this->_kolab = &new Kolab();
         }
-
-        $this->_kolab = new Kolab();
-
         return $this->_kolab->open($this->_calendar);
     }
 
@@ -101,11 +100,30 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
         return is_array($events) ? $events : array();
     }
 
+    /**
+     * Checks if the event's UID already exists and returns all event
+     * ids with that UID.
+     *
+     * @param string $uid          The event's uid.
+     * @param string $calendar_id  Calendar to search in.
+     *
+     * @return mixed  Returns a string with event_id or false if not found.
+     */
+    function exists($uid, $calendar_id = null)
+    {
+        // Don't use calendar id here.
+        if (is_a($this->_kolab->loadObject($uid), 'PEAR_Error')) {
+            return false;
+        }
+
+        return $uid;
+    }
+
     function listEvents($startDate = null, $endDate = null)
     {
-        // We don't perform any checking on $startDate and $endDate, as that
-        // has the potential to leave out recurring event instances.
-
+        // We don't perform any checking on $startDate and $endDate,
+        // as that has the potential to leave out recurring event
+        // instances.
         $events = array();
 
         $msg_list = $this->_kolab->listObjects();
@@ -123,7 +141,7 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
                 return $result;
             }
 
-            $events[] = $this->_kolab->getUID();
+            $events[$this->_kolab->getUID()] = $this->_kolab->getUID();
         }
 
         return $events;
@@ -132,7 +150,8 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
     function &getEvent($eventID = null)
     {
         if (is_null($eventID)) {
-            return $event = &new Kronolith_Event_kolab($this);
+            $event = &new Kronolith_Event_kolab($this);
+            return $event;
         }
 
         $result = $this->_kolab->loadObject($eventID);
@@ -140,25 +159,31 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
             return $result;
         }
 
-        return $event = &new Kronolith_Event_kolab($this, true);
+        $event = &new Kronolith_Event_kolab($this);
+        $event->fromDriver($this);
+
+        return $event;
     }
 
     function &getByUID($uid)
     {
-        return PEAR::raiseError('Not supported');
+        $this->open(Auth::getAuth());
+
+        return $this->getEvent($uid);
     }
 
     function saveEvent($event)
     {
         $k = &$this->_kolab;
 
-        if ($event->isStored()) {
+        $edit = false;
+        if ($event->isStored() || $event->exists()) {
             $uid = $event->getID();
 
             $result = $k->loadObject($uid);
-            if (is_a($result, 'PEAR_Error')) {
-                return $result;
-            }
+            //No error check here, already done in exists()
+
+            $edit = true;
         } else {
             if ($event->getID()) {
                 $uid = $event->getID();
@@ -180,8 +205,13 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
         $k->setElemStr($organizer, 'smtp-address', $event->getCreatorID());
 
         $k->setVal('alarm', $event->getAlarm());
-        $k->setVal('start-date', Kolab::encodeDateTime($event->start->timestamp()));
-        $k->setVal('end-date', Kolab::encodeDateTime($event->end->timestamp()));
+        if ($event->isAllDay()) {
+            $k->setVal('start-date', Kolab::encodeDate($event->start->timestamp()));
+            $k->setVal('end-date', Kolab::encodeDate($event->end->timestamp()-24*60*60));
+        } else {
+            $k->setVal('start-date', Kolab::encodeDateTime($event->start->timestamp()));
+            $k->setVal('end-date', Kolab::encodeDateTime($event->end->timestamp()));
+        }
 
         switch ($event->status) {
         case KRONOLITH_STATUS_CANCELLED:
@@ -305,6 +335,12 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
             return $result;
         }
 
+        /* Notify about the changed event. */
+        $result = Kronolith::sendNotification($event, $edit ? 'edit' : 'add');
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
+
         return $uid;
     }
 
@@ -355,7 +391,20 @@ class Kronolith_Driver_kolab extends Kronolith_Driver {
      */
     function deleteEvent($eventID)
     {
-        return $this->_kolab->removeObjects($eventID);
+        /* Fetch the event for later use. */
+        $event = &$this->getEvent($eventID);
+
+        /* Delete the event. */
+        $deleted = $this->_kolab->removeObjects($eventID);
+        if (!$deleted || is_a($deleted, 'PEAR_Error')) {
+            return $deleted;
+        }
+
+        /* Notify about the deleted event. */
+        $result = Kronolith::sendNotification($event, 'delete');
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
     }
 
 }
@@ -379,7 +428,7 @@ class Kronolith_Event_kolab extends Kronolith_Event {
 
         $this->alarm = $k->getVal('alarm');
         $this->start = &new Horde_Date(Kolab::decodeDateOrDateTime($k->getVal('start-date')));
-        $this->end = &new Horde_Date(Kolab::decodeDateOrDateTime($k->getVal('end-date')));
+        $this->end = &new Horde_Date(Kolab::decodeFullDayDate($k->getVal('end-date')));
         $this->durMin = ($this->end->timestamp() - $this->start->timestamp()) / 60;
 
         $status = $k->getVal('show-time-as');
